@@ -20,15 +20,15 @@ from math import log10
 from . import Mp3Exception
 
 from ..utils.binfuncs import bytes2bin, bytes2dec, bin2dec
-from .. import core
+from .. import compat
 
 from ..utils.log import getLogger
 log = getLogger(__name__)
 
 
 def isValidHeader(header):
-    '''Determine if ``header`` (an integer, 4 bytes compared) is a valid mp3
-    frame header.'''
+    """Determine if ``header`` (an integer, 4 bytes compared) is a valid mp3
+    frame header."""
     # Test for the mp3 frame sync: 11 set bits.
     sync = (header >> 16)
     if sync & 0xffe0 != 0xffe0:
@@ -65,26 +65,43 @@ def isValidHeader(header):
 
     return True
 
+
 def findHeader(fp, start_pos=0):
-    '''Locate the first mp3 header in file stream ``fp`` starting a offset
+    """Locate the first mp3 header in file stream ``fp`` starting a offset
     ``start_pos`` (defaults to 0). Returned is a 3-tuple containing the offset
     where the header was found, the header as an integer, and the header as 4
     bytes. If no header is found header_int will equal 0.
-    '''
+    """
+
+    def isBOM(buffer, pos):
+        try:
+            return 254 in (buffer[pos + 1], buffer[pos - 1])
+        except IndexError:
+            return False
 
     def find_sync(fp, start_pos=0):
-        CHUNK_SIZE = 8192 # Measured as optimal
+        CHUNK_SIZE = 8192  # Measured as optimal
 
         fp.seek(start_pos)
         data = fp.read(CHUNK_SIZE)
 
         while data:
-            sync_pos = data.find(b'\xff', 0)
-            if sync_pos >= 0:
-                header = data[sync_pos:sync_pos + 4]
-                if len(header) == 4:
-                    return (start_pos + sync_pos, header)
+            pos = 0
+            while pos >= 0 and pos < CHUNK_SIZE:
+                pos = data.find(b"\xff", pos)
+                if pos == -1:
+                    break
+
+                if not isBOM(data, pos):
+                    header = data[pos:pos + 4]
+                    if len(header) == 4:
+                        return (start_pos + pos, header)
+
+                pos += 1
+
+            start_pos += len(data)
             data = fp.read(CHUNK_SIZE)
+
         return (None, None)
 
     sync_pos, header_bytes = find_sync(fp, start_pos)
@@ -97,12 +114,11 @@ def findHeader(fp, start_pos=0):
 
 
 def timePerFrame(mp3_header, vbr):
-    '''Computes the number of seconds per mp3 frame. It can be used to
+    """Computes the number of seconds per mp3 frame. It can be used to
     compute overall playtime and bitrate. The mp3 layer and sample
     rate from ``mp3_header`` are used to compute the number of seconds
     (fractional float point value) per mp3 frame. Be sure to set ``vbr`` True
-    when dealing with VBR, otherwise playtimes may be incorrect.'''
-
+    when dealing with VBR, otherwise playtimes may be incorrect."""
     # https://bitbucket.org/nicfit/eyed3/issue/32/mp3audioinfotime_secs-incorrect-for-mpeg2
     if mp3_header.version >= 2.0 and vbr:
         row = _mp3VersionKey(mp3_header.version)
@@ -113,14 +129,14 @@ def timePerFrame(mp3_header, vbr):
 
 
 def compute_time_per_frame(mp3_header):
-    '''Deprecated, use timePerFrame instead.'''
+    """Deprecated, use timePerFrame instead."""
     import warnings
     warnings.warn("Use timePerFrame instead", DeprecationWarning, stacklevel=2)
     return timePerFrame(mp3_header, False)
 
 
 class Mp3Header:
-    '''Header container for MP3 frames.'''
+    """Header container for MP3 frames."""
     def __init__(self, header_data=None):
         self.version = None
         self.layer = None
@@ -166,8 +182,8 @@ class Mp3Header:
 
         # Obtain sampling frequency.
         sample_bits = (header >> 10) & 0x3
-        self.sample_freq = SAMPLE_FREQ_TABLE[sample_bits]\
-                                            [_mp3VersionKey(self.version)]
+        self.sample_freq = \
+            SAMPLE_FREQ_TABLE[sample_bits][_mp3VersionKey(self.version)]
         if not self.sample_freq:
             raise Mp3Exception("Illegal MPEG sampling frequency")
 
@@ -181,14 +197,14 @@ class Mp3Header:
             bit_rate_col = 2
         elif int(self.version) == 2 and self.layer == 1:
             bit_rate_col = 3
-        elif int(self.version) == 2 and (self.layer == 2 or \
+        elif int(self.version) == 2 and (self.layer == 2 or
                                          self.layer == 3):
             bit_rate_col = 4
         else:
-            raise Mp3Exception("Mp3 version %f and layer %d is an invalid "\
-                              "combination" % (self.version, self.layer))
+            raise Mp3Exception("Mp3 version %f and layer %d is an invalid "
+                               "combination" % (self.version, self.layer))
         self.bit_rate = BIT_RATE_TABLE[bit_rate_row][bit_rate_col]
-        if self.bit_rate == None:
+        if self.bit_rate is None:
             raise Mp3Exception("Invalid bit rate")
         # We know know the bit rate specified in this frame, but if the file
         # is VBR we need to obtain the average from the Xing header.
@@ -232,14 +248,14 @@ class Mp3Header:
 
         br = self.bit_rate * 1000
         sf = self.sample_freq
-        p  = self.padding
+        p = self.padding
         if self.layer == 1:
             # Layer 1 uses 32 bit slots for padding.
-            p  = self.padding * 4
+            p = self.padding * 4
             self.frame_length = int((((12 * br) / sf) + p) * 4)
         else:
             # Layer 2 and 3 uses 8 bit slots for padding.
-            p  = self.padding * 1
+            p = self.padding * 1
             self.frame_length = int(((144 * br) / sf) + p)
 
         # Dump the state.
@@ -257,6 +273,7 @@ class Mp3Header:
         log.debug("MPEG emphasis: " + str(self.emphasis))
         log.debug("MPEG frame length: " + str(self.frame_length))
 
+
 class VbriHeader(object):
     def __init__(self):
         self.vbr = True
@@ -282,10 +299,10 @@ class VbriHeader(object):
         self.delay = bin2dec(bytes2bin(frame[offset:offset + 2]))
         offset += 2
 
-        self.quality =    bin2dec(bytes2bin(frame[offset:offset + 2]))
+        self.quality = bin2dec(bytes2bin(frame[offset:offset + 2]))
         offset += 2
 
-        self.num_bytes =  bin2dec(bytes2bin(frame[offset:offset + 4]))
+        self.num_bytes = bin2dec(bytes2bin(frame[offset:offset + 4]))
         offset += 4
 
         self.num_frames = bin2dec(bytes2bin(frame[offset:offset + 4]))
@@ -293,8 +310,9 @@ class VbriHeader(object):
 
         return True
 
+
 class XingHeader:
-    '''Header class for the Xing header extensions.'''
+    """Header class for the Xing header extensions."""
 
     def __init__(self):
         self.numFrames = int()
@@ -308,9 +326,9 @@ class XingHeader:
     # frame, and false otherwise.
     def decode(self, frame):
         # mp3 version
-        version = (ord(frame[1]) >> 3) & 0x1
+        version = (compat.byteOrd(frame[1]) >> 3) & 0x1
         # channel mode.
-        mode = (ord(frame[3]) >> 6) & 0x3
+        mode = (compat.byteOrd(frame[3]) >> 6) & 0x3
 
         # Find the start of the Xing header.
         if version:
@@ -324,9 +342,9 @@ class XingHeader:
                 pos = 17 + 4
             else:
                 pos = 9 + 4
-        head = frame[pos:pos+4]
-        self.vbr = (head == 'Xing') and True or False
-        if head not in ['Xing', 'Info']:
+        head = frame[pos:pos + 4]
+        self.vbr = (head == b'Xing') and True or False
+        if head not in [b'Xing', b'Info']:
             return False
         log.debug("%s header detected @ %x" % (head, pos))
         pos += 4
@@ -357,13 +375,14 @@ class XingHeader:
             log.debug("%s TOC (100 bytes): NOT PRESENT" % head)
 
         # Read vbr scale header flag and value if present
-        if headFlags & VBR_SCALE_FLAG and head == 'Xing':
+        if headFlags & VBR_SCALE_FLAG and head == b'Xing':
             self.vbrScale = bin2dec(bytes2bin(frame[pos:pos + 4]))
             pos += 4
             log.debug("%s vbrScale: %d" % (head, self.vbrScale))
 
         return True
 
+
 ##
 # \brief Mp3 Info tag (AKA LAME Tag)
 #
@@ -469,24 +488,24 @@ class LameHeader(dict):
       0x8201, 0x42C0, 0x4380, 0x8341, 0x4100, 0x81C1, 0x8081, 0x4040]
 
     ENCODER_FLAGS = {
-      'NSPSYTUNE'   : 0x0001,
-      'NSSAFEJOINT' : 0x0002,
-      'NOGAP_NEXT'  : 0x0004,
-      'NOGAP_PREV'  : 0x0008,}
+      'NSPSYTUNE': 0x0001,
+      'NSSAFEJOINT': 0x0002,
+      'NOGAP_NEXT': 0x0004,
+      'NOGAP_PREV': 0x0008}
 
     PRESETS = {
-      0:    'Unknown',
+      0: 'Unknown',
       # 8 to 320 are reserved for ABR bitrates
-      410:  'V9',
-      420:  'V8',
-      430:  'V7',
-      440:  'V6',
-      450:  'V5',
-      460:  'V4',
-      470:  'V3',
-      480:  'V2',
-      490:  'V1',
-      500:  'V0',
+      410: 'V9',
+      420: 'V8',
+      430: 'V7',
+      440: 'V6',
+      450: 'V5',
+      460: 'V4',
+      470: 'V3',
+      480: 'V2',
+      490: 'V1',
+      500: 'V0',
       1000: 'r3mix',
       1001: 'standard',
       1002: 'extreme',
@@ -494,25 +513,25 @@ class LameHeader(dict):
       1004: 'standard/fast',
       1005: 'extreme/fast',
       1006: 'medium',
-      1007: 'medium/fast',}
+      1007: 'medium/fast'}
 
     REPLAYGAIN_NAME = {
       0: 'Not set',
       1: 'Radio',
-      2: 'Audiofile',}
+      2: 'Audiofile'}
 
     REPLAYGAIN_ORIGINATOR = {
-      0:   'Not set',
-      1:   'Set by artist',
-      2:   'Set by user',
-      3:   'Set automatically',
-      100: 'Set by simple RMS average',}
+      0: 'Not set',
+      1: 'Set by artist',
+      2: 'Set by user',
+      3: 'Set automatically',
+      100: 'Set by simple RMS average'}
 
     SAMPLE_FREQUENCIES = {
       0: '<= 32 kHz',
       1: '44.1 kHz',
       2: '48 kHz',
-      3: '> 48 kHz',}
+      3: '> 48 kHz'}
 
     STEREO_MODES = {
       0: 'Mono',
@@ -522,26 +541,26 @@ class LameHeader(dict):
       4: 'Force',
       5: 'Auto',
       6: 'Intensity',
-      7: 'Undefined',}
+      7: 'Undefined'}
 
     SURROUND_INFO = {
       0: 'None',
       1: 'DPL encoding',
       2: 'DPL2 encoding',
       3: 'Ambisonic encoding',
-      8: 'Reserved',}
+      8: 'Reserved'}
 
     VBR_METHODS = {
-      0:  'Unknown',
-      1:  'Constant Bitrate',
-      2:  'Average Bitrate',
-      3:  'Variable Bitrate method1 (old/rh)',
-      4:  'Variable Bitrate method2 (mtrh)',
-      5:  'Variable Bitrate method3 (mt)',
-      6:  'Variable Bitrate method4',
-      8:  'Constant Bitrate (2 pass)',
-      9:  'Average Bitrate (2 pass)',
-      15: 'Reserved',}
+      0: 'Unknown',
+      1: 'Constant Bitrate',
+      2: 'Average Bitrate',
+      3: 'Variable Bitrate method1 (old/rh)',
+      4: 'Variable Bitrate method2 (mtrh)',
+      5: 'Variable Bitrate method3 (mt)',
+      6: 'Variable Bitrate method4',
+      8: 'Constant Bitrate (2 pass)',
+      9: 'Average Bitrate (2 pass)',
+      15: 'Reserved'}
 
     def __init__(self, frame):
         """Read the LAME info tag.
@@ -549,17 +568,17 @@ class LameHeader(dict):
         """
         self.decode(frame)
 
-    def _crc16(self, data, val = 0):
+    def _crc16(self, data, val=0):
         """Compute a CRC-16 checksum on a data stream."""
-        for c in data:
+        for c in compat.byteiter(data):
             val = self._crc16_table[ord(c) ^ (val & 0xff)] ^ (val >> 8)
         return val
 
     def decode(self, frame):
         """Decode the LAME info tag."""
         try:
-            pos = frame.index("LAME")
-        except:
+            pos = frame.index(b"LAME")
+        except:                                                     # noqa: B901
             return
 
         log.debug('Lame info tag found at position %d' % pos)
@@ -571,7 +590,8 @@ class LameHeader(dict):
 
         try:
             # Encoder short VersionString, 9 bytes
-            self['encoder_version'] = lamever = frame[pos:pos + 9].rstrip()
+            self['encoder_version'] = \
+                compat.unicode(frame[pos:pos + 9].rstrip(), "latin1")
             log.debug('Lame Encoder Version: %s' % self['encoder_version'])
             pos += 9
 
@@ -608,13 +628,16 @@ class LameHeader(dict):
                 name = bin2dec(bytes2bin(frame[pos:pos + 2])[:3])
                 orig = bin2dec(bytes2bin(frame[pos:pos + 2])[3:6])
                 sign = bin2dec(bytes2bin(frame[pos:pos + 2])[6:7])
-                adj  = bin2dec(bytes2bin(frame[pos:pos + 2])[7:]) / 10.0
+                adj = bin2dec(bytes2bin(frame[pos:pos + 2])[7:]) / 10.0
                 if sign:
                     adj *= -1
-                # XXX Lame 3.95.1 and above use 89dB as a reference instead of
+                # FIXME Lame 3.95.1 and above use 89dB as a reference instead of
                 # 83dB as defined by the Replay Gain spec. Should this be
-                # compensated for?  if lamever[:4] == 'LAME' and
-                # lamevercmp(lamever[4:], '3.95') > 0: adj -= 6
+                # compensated for?
+                # lamever =self['encoder_version']
+                # if (lamever[:4] == 'LAME' and
+                #       lamevercmp(lamever[4:], '3.95') > 0):
+                #     adj -= 6
                 if orig:
                     name = self.REPLAYGAIN_NAME.get(name, 'Unknown')
                     orig = self.REPLAYGAIN_ORIGINATOR.get(orig, 'Unknown')
@@ -715,7 +738,7 @@ class LameHeader(dict):
             pos += 2
 
             # CRC-16 of Info Tag, 2 bytes
-            self['infotag_crc'] = lamecrc # we read this earlier
+            self['infotag_crc'] = lamecrc  # we read this earlier
             log.debug('Lame Info Tag CRC: %04X' % self['infotag_crc'])
             pos += 2
         except IndexError:
@@ -748,6 +771,7 @@ class LameHeader(dict):
                 nogap.append('after')
         return encoder_flags, nogap
 
+
 ##
 # \brief Compare LAME version strings.
 #
@@ -762,7 +786,7 @@ def lamevercmp(x, y):
     y = y.ljust(5)
     if x[:5] == y[:5]:
         return 0
-    ret = cmp(x[:4], y[:4])
+    ret = compat.cmp(x[:4], y[:4])
     if ret:
         return ret
     xmaj, xmin = x.split('.')[:2]
@@ -779,37 +803,38 @@ def lamevercmp(x, y):
         return 1
     if y[4] == ' ':
         return -1
-    return cmp(x[4], y[4])
+    return compat.cmp(x[4], y[4])
 
-#                   MPEG1  MPEG2  MPEG2.5
+
+#                     MPEG1  MPEG2  MPEG2.5
 SAMPLE_FREQ_TABLE = ((44100, 22050, 11025),
                      (48000, 24000, 12000),
                      (32000, 16000, 8000),
-                     (None,  None,  None))
+                     (None, None, None))
 
 #              V1/L1  V1/L2 V1/L3 V2/L1 V2/L2&L3
-BIT_RATE_TABLE = ((0,    0,    0,    0,    0),
-                  (32,   32,   32,   32,   8),
-                  (64,   48,   40,   48,   16),
-                  (96,   56,   48,   56,   24),
-                  (128,  64,   56,   64,   32),
-                  (160,  80,   64,   80,   40),
-                  (192,  96,   80,   96,   48),
-                  (224,  112,  96,   112,  56),
-                  (256,  128,  112,  128,  64),
-                  (288,  160,  128,  144,  80),
-                  (320,  192,  160,  160,  96),
-                  (352,  224,  192,  176,  112),
-                  (384,  256,  224,  192,  128),
-                  (416,  320,  256,  224,  144),
-                  (448,  384,  320,  256,  160),
+BIT_RATE_TABLE = ((0,    0,    0,    0,    0),                          # noqa
+                  (32,   32,   32,   32,   8),                          # noqa
+                  (64,   48,   40,   48,   16),                         # noqa
+                  (96,   56,   48,   56,   24),                         # noqa
+                  (128,  64,   56,   64,   32),                         # noqa
+                  (160,  80,   64,   80,   40),                         # noqa
+                  (192,  96,   80,   96,   48),                         # noqa
+                  (224,  112,  96,   112,  56),                         # noqa
+                  (256,  128,  112,  128,  64),                         # noqa
+                  (288,  160,  128,  144,  80),                         # noqa
+                  (320,  192,  160,  160,  96),                         # noqa
+                  (352,  224,  192,  176,  112),                        # noqa
+                  (384,  256,  224,  192,  128),                        # noqa
+                  (416,  320,  256,  224,  144),                        # noqa
+                  (448,  384,  320,  256,  160),                        # noqa
                   (None, None, None, None, None))
 
 # Rows 1 and 2 (mpeg 2.x) are only used for those versions *and* VBR.
 #                                  L1   L2   L3
-SAMPLES_PER_FRAME_TABLE = ((None, 384, 1152, 1152), # MPEG 1
-                           (None, 384, 1152, 576),  # MPEG 2
-                           (None, 384, 1152, 576),  # MPEG 2.5
+SAMPLES_PER_FRAME_TABLE = ((None, 384, 1152, 1152),  # MPEG 1
+                           (None, 384, 1152, 576),   # MPEG 2
+                           (None, 384, 1152, 576),   # MPEG 2.5
                           )
 
 # Emphasis constants
@@ -818,22 +843,22 @@ EMPHASIS_5015 = "50/15 ms"
 EMPHASIS_CCIT = "CCIT J.17"
 
 # Mode constants
-MODE_STEREO              = "Stereo"
-MODE_JOINT_STEREO        = "Joint stereo"
+MODE_STEREO = "Stereo"
+MODE_JOINT_STEREO = "Joint stereo"
 MODE_DUAL_CHANNEL_STEREO = "Dual channel stereo"
-MODE_MONO                = "Mono"
+MODE_MONO = "Mono"
 
 # Xing flag bits
-FRAMES_FLAG    = 0x0001
-BYTES_FLAG     = 0x0002
-TOC_FLAG       = 0x0004
+FRAMES_FLAG = 0x0001
+BYTES_FLAG = 0x0002
+TOC_FLAG = 0x0004
 VBR_SCALE_FLAG = 0x0008
 
 
 def _mp3VersionKey(version):
-    '''Map mp3 version float to a data structure index.
+    """Map mp3 version float to a data structure index.
     1 -> 0, 2 -> 1, 2.5 -> 2
-    '''
+    """
     key = None
     if version == 2.5:
         key = 2
